@@ -1,6 +1,8 @@
+from django import forms
 from django.utils import simplejson as json
 from django.db.models.fields.subclassing import Creator
 from django.db.models import TextField
+from django.utils.encoding import smart_unicode
 
 from south.modelsinspector import add_introspection_rules
 
@@ -58,6 +60,26 @@ class AppDataField(TextField):
         super(AppDataField, self).validate(value, model_instance)
         value.validate(model_instance)
 
+    def value_to_string(self, obj):
+        value = self._get_val_from_obj(obj)
+
+        if isinstance(value, AppDataContainerFactory):
+            value = value.serialize()
+        if isinstance(value, dict):
+            value = json.dumps(value)
+
+        return smart_unicode(value)
+
+
+class ListModelMultipleChoiceField(forms.ModelMultipleChoiceField):
+    """
+    A ModelMultipleChoiceField that cleans to a list instead of a QuerySet
+
+    Use this on AppDataForms rather than a ModelMultipleChoiceField to make it
+    possible to manipulate the app data field like a list.
+    """
+    def clean(self, value):
+        value = super(ListModelMultipleChoiceField, self).clean(value)
+        return value and list(value) or value
 
 add_introspection_rules([], ["^app_data\.fields\.AppDataField"])
-
