@@ -4,6 +4,8 @@ except ImportError:
     methodcaller = lambda name: lambda o: getattr(o, name)()
 
 from django.forms.forms import NON_FIELD_ERRORS, Form
+from django.forms.formsets import formset_factory
+from django.forms.models import modelform_factory, _get_foreign_key, BaseInlineFormSet, BaseModelFormSet
 
 class AppDataForm(Form):
     def __init__(self, app_container, data=None, files=None, fields=(), exclude=(), **kwargs):
@@ -77,7 +79,10 @@ class MultiForm(object):
         self.app_forms = {}
         app_container = getattr(self.model_form.instance, self.app_data_field)
         for label, label_opts in self.get_app_form_opts().iteritems():
-            self.app_forms[label] = app_container[label].get_form(data, files, prefix=label, **label_opts)
+            prefix = label
+            if self.model_form.prefix:
+                prefix = '%s-%s' % (self.model_form.prefix, prefix)
+            self.app_forms[label] = app_container[label].get_form(data, files, prefix=prefix, **label_opts)
 
     @classmethod
     def get_app_form_opts(cls):
@@ -119,6 +124,33 @@ class MultiForm(object):
         cls.app_form_opts[label] = None
 
     # properties delegated to model_form
+    @property
+    def _get_validation_exclusions(self):
+        return self.model_form._get_validation_exclusions
+
+    @property
+    def cleaned_data(self):
+        return self.model_form.cleaned_data
+
+    @property
+    def _meta(self):
+        # user by BaseInlineFormSet.add_fields
+        return self.model_form._meta
+
+    @property
+    def fields(self):
+        # user by BaseModelFormSet.add_fields
+        return self.model_form.fields
+
+    @property
+    def _raw_value(self):
+        # used by FormSet._should_delete_form
+        return self.model_form._raw_value
+
+    @property
+    def instance(self):
+        return self.model_form.instance
+
     @property
     def media(self):
         return self.model_form.media
@@ -154,6 +186,9 @@ class MultiForm(object):
     def is_valid(self):
         return all(map(methodcaller('is_valid'), self._get_all_forms()))
 
+    def has_changed(self):
+        return any(map(methodcaller('has_changed'), self._get_all_forms()))
+
     def __getitem__(self, name):
         # provide access to app.field as well
         app = None
@@ -206,9 +241,29 @@ class MultiForm(object):
         # save the model itself
         return self.model_form.save(**kwargs)
 
-def multiform_factory(model_form, base_class=MultiForm, app_data_field='app_data', name=None, **form_opts):
+
+def multiform_factory(model, multiform=MultiForm, app_data_field='app_data', name=None, form_opts={}, **kwargs):
+    model_form = modelform_factory(model, **kwargs)
     name = name or '%sWithAppDataForm' % model_form._meta.model.__name__
     return type(
-        name, (base_class, ),
+        name, (multiform, ),
         {'ModelForm': model_form, 'app_data_field': app_data_field, '_app_form_opts': form_opts}
     )
+
+def multiformset_factory(model, multiform=MultiForm, app_data_field='app_data', name=None, form_opts={},
+                         formset=BaseModelFormSet, extra=3, can_order=False, can_delete=True, max_num=None,
+                         **kwargs):
+    multiform = multiform_factory(model, multiform, app_data_field, name, form_opts, **kwargs)
+    FormSet = formset_factory(multiform, formset=formset, extra=extra, can_order=can_order, can_delete=can_delete, max_num=max_num)
+    FormSet.model = model
+    return FormSet
+
+def multiinlineformset_factory(parent_model, model, multiform=MultiForm, app_data_field='app_data', name=None, form_opts={},
+                                formset=BaseInlineFormSet, fk_name=None, **kwargs):
+    fk = _get_foreign_key(parent_model, model, fk_name=fk_name)
+    if fk.unique:
+        kwargs['max_num'] = 1
+
+    FormSet = multiformset_factory(model, multiform, app_data_field, name, form_opts, formset=formset, **kwargs)
+    FormSet.fk = fk
+    return FormSet
