@@ -21,7 +21,7 @@ SECRET_KEY = 'very-secret'
 DATABASES = {
     'default': {
         'ENGINE': 'django.db.backends.sqlite3',
-        'NAME': ':memory:',
+        'NAME': '/tmp/app_data.db',
     }
 }
 
