@@ -1,10 +1,9 @@
 from datetime import date
 
 from django import forms
-from django.conf import settings
 from django.forms.models import ModelChoiceField, modelform_factory
 
-from app_data.forms import multiform_factory, MultiForm
+from app_data.forms import multiform_factory, MultiForm, multiformset_factory
 from app_data.registry import app_registry
 from app_data.containers import AppDataContainer, AppDataForm
 
@@ -32,11 +31,35 @@ class TestMultiForm(AppDataTestCase):
         app_registry.register('myapp', MyAppContainer)
         app_registry.register('myapp2', AppDataContainer.from_form(self.MyForm2))
 
+    def test_multi_form_can_work_with_formsets(self):
+        FormSet = multiformset_factory(Article, form_opts={'myapp': {}})
+        data = {
+            'fs-TOTAL_FORMS': '1',
+            'fs-INITIAL_FORMS': '0',
+
+            'fs-0-myapp-title': 'First',
+            'fs-0-myapp-publish_from': '2010-11-12',
+        }
+        formset = FormSet(data, prefix='fs')
+
+        tools.assert_true(formset.is_valid())
+        formset.save()
+        tools.assert_equals(1, Article.objects.count())
+        art = Article.objects.all()[0]
+        tools.assert_equals(
+            {
+                'myapp': {
+                    'publish_from': '2010-11-12',
+                    'publish_to': None,
+                    'related_article': None,
+                    'title': u'First'
+                },
+            },
+            art.app_data
+        )
+
     def test_multi_form_saves_all_the_forms(self):
-        ModelForm = modelform_factory(Article)
-        MF = multiform_factory(ModelForm)
-        MF.add_form('myapp')
-        MF.add_form('myapp2')
+        MF = multiform_factory(Article, form_opts={'myapp': {}, 'myapp2': {}})
         data = {
             'myapp-title': 'First',
             'myapp-publish_from': '2010-11-12',
@@ -60,8 +83,7 @@ class TestMultiForm(AppDataTestCase):
         )
 
     def test_form_can_be_added_to_parent(self):
-        ModelForm = modelform_factory(Article)
-        MF = multiform_factory(ModelForm, base_class=self.MyMultiForm)
+        MF = multiform_factory(Article, multiform=self.MyMultiForm)
         self.MyMultiForm.add_form('myapp', {})
         data = {
             'myapp-title': 'First',
@@ -84,8 +106,7 @@ class TestMultiForm(AppDataTestCase):
         )
 
     def test_form_can_be_added(self):
-        ModelForm = modelform_factory(Article)
-        MF = multiform_factory(ModelForm)
+        MF = multiform_factory(Article)
         MF.add_form('myapp', {})
         data = {
             'myapp-title': 'First',
@@ -116,8 +137,7 @@ class TestMultiForm(AppDataTestCase):
         tools.assert_equals({}, MultiForm.app_form_opts)
 
     def test_form_can_be_removed(self):
-        ModelForm = modelform_factory(Article)
-        MF = multiform_factory(ModelForm, myapp={})
+        MF = multiform_factory(Article, form_opts={'myapp': {}})
         MF.remove_form('myapp')
         data = {
             'myapp-title': 'First',
